--########################################################################
--
--                               CIRCUITO TOP
--
--   FERNANDO GEHM MORAES	03/agosto/2005
--   NEY LAERT VILAR CALAZANS	16/junho/2006 - Adaptao para D2SB/DIO4
--				e acrscimo de entradas para controles
--   NEY LAERT VILAR CALAZANS	27/junho/2006 - Adaptao para simulao,
--				generalizando para 3 sadas de 8 bits e para duas 
-- 				entradas de 8 bits
--
--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_unsigned.all;   -- uso conv_integer
use IEEE.Std_Logic_arith.all;
 
entity top is
          port( ck :		in  std_logic;
                reset:		in  std_logic; -- Ney: active-high reset
				halt :    out std_logic;
                input1:		in  std_logic_vector(7 downto 0);
                input2:		in  std_logic_vector(7 downto 0); 
                output1:	out std_logic_vector(7 downto 0);
                output2:	out std_logic_vector(7 downto 0);
                output3:	out std_logic_vector(7 downto 0)
             );
end top; 

architecture top of top is
                                      
   signal hold, ce, rw, ceCPU, rwCPU, ceES, rwES: std_logic;
   signal address, 
   	  datainCPU,
   	  dataoutCPU,
   	  dataoutRAM,
   	  d1_CPU, -- output to two displays
   	  d2_CPU  -- output to two displays
   	  : std_logic_vector(7 downto 0);
  
   signal d1,d2,d3,d4: std_logic_vector(6 downto 0);

   signal  CLK_DIV : std_logic:='0';

begin

   -- The 50MHz clock of the board is too fast for the Clepatra processor 
   -- to run without careful logic/physical synthesis. To avoid problems
   -- the process below simply divides the clock by 2 (from 50MHz to 25MHz)
   -- A better option would be to optimize the synthesis to make it run at 
   -- 50MHz, or at least to use a DLL to divide the clock properly.
   -- For teaching purposes this process is good enough
   --
   -- Added by Ney (18/06/2006)
   --
   process (ck)
   begin
	if ck'event and ck='1' then
		CLK_DIV <= not CLK_DIV;
	end if;	
   end process;

   -------------------------------------------------------------------------------
   -- Aqui, insere-se a instncia o processador Clepatra
   --
   cpuC : entity work.cleopatra port map
               ( clock=>CLK_DIV, reset=>reset, halt=>halt, hold=>hold,
                 ce=>ce, rw=>rw, address=>address ,datain=>datainCPU, dataout=>dataoutCPU);
   -- Para garantir que processador no  impedido de progredir, coloca-se hold em 0
   hold <= '0';
	
   -------------------------------------------------------------------------------
   -- Aqui, insere-se a instncia da memria, j com o programa inicializado
   --
   ramC: entity work.RAM_mem port map
               ( clock=>CLK_DIV, ce=>ceCPU,rw=>rwCPU, address=>address,
               datain=>dataoutCPU, dataout=>dataoutRAM);
 
 
   -------------------------------------------------------------------------------
   -- instancia o decodificador que define se  acesso  memria ou a entrada/dada
   --
   deco: entity work.es_decoder port map
             ( ce => ce, rw=>rw, address => address,
               ceCPU=>ceCPU, rwCPU=>rwCPU, ceES=>ceES, rwES=>rwES);
 
   -------------------------------------------------------------------------------
   -- Comando que captura o dado de algum dispositivo externo  CPU Clepatra.
   -- Este comando  disparado por uma instruo LDA executada pela Clepatra.
   --
   -- O dado lido pode vir da memria ou de algum disppositivo de entrada, quais
   -- sejam input1 e input2. Recomenda-se usar uma destas para entrada de dados e 
   -- a outra para entrada de sinais de controle.
   -- Mapa de leitura:
   --	Leitura de endereos 00h a 0FBh => leitura de posio da memria principal
   --	Leitura de FC => captura valor de input1, colocando no Acumulador
   --	Leitura de FD => captura valor de input2, colocando no Acumulador
   --	Leitura de FE => sem efeito nesta implementao
   --	Leitura de FF => sem efeito nesta implementao
   --
   datainCPU <= dataoutRAM 	when ceCPU='1' else 
              input1		when ceES='1' and address=x"FC" else
              input2		when ceES='1' and address=x"FD" else
              -- Coloca no acumulador o valor das chaves
              (others=>'Z');
    

   -------------------------------------------------------------------------------
   -- Processo que captura o dado escrito pela CPU em dispositivos de sada 
   -- 
   -- output1, output2 e output3 so trs registradores de 8 bits que recebem
   --	dados via instrues STA do processador e disponibilizam sua sadas em 
   --	dispositivos de E/S (displays e leds).
   -- Mapa de escrita:
   --	Escrita em endereos 00h a 0FBh => escrita em posio da memria principal
   --	Escrita em FC => sem efeito nesta implementao
   --	Escrita em FD => efeito em output1
   --	Escrita em FE => efeito em output2
   --	Escrita em FF => efeito em output3
   --
   process (CLK_DIV, reset)
    begin
         if reset='1' then
              output1  <= (others=>'0');
              output2  <= (others=>'0');
              output3  <= (others=>'0');
         elsif CLK_DIV'event and CLK_DIV='1' then
              if ceES='1' and  rwES='0' then 
                   -- seno, envia para os leds o contedo do acumulador
                   if address=x"FD" then output1	<= dataoutCPU; end if; 
                   if address=x"FE" then output2 	<= dataoutCPU; end if;
                   if address=x"FF" then output3	<= dataoutCPU; end if;  
              end if;  
         end if;
   end process;
 
end top;
